/******************************************************************************

Copyright 2023, Beijing ESWIN Computing Technology Co., Ltd.. All rights reserved.

******************************************************************************
File Name : sample_common.h

*******************************************************************************/
#ifndef __ES_CIPHER_SAMPLE_COMMON_H__
#define __ES_CIPHER_SAMPLE_COMMON_H__
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include <essdk/es_type.h>
#include <essdk/es_sys_memory.h>
#include <essdk/es_cipher.h>

#define ES_ERR_CIPHER(format, arg...)     printf( "\033[0;1;31m" format "\033[0m", ## arg)
#define ES_INFO_CIPHER(format, arg...)    printf( "\033[0;1;32m" format "\033[0m", ## arg)


#define RSA2048_PUBLICKEY_LEN 256
#define RSA2048_PRIKEY_LEN 256

/* RSA2048 public key structure */
typedef struct __rsa2048_pubkey_st
{
  ES_U32 keylen;
  ES_U32 exponent;
  ES_U8 mod[RSA2048_PUBLICKEY_LEN];
} rsa2048_pubkey_t;

/* RSA2048 private key structure */
typedef struct __rsa2048_privkey_st
{
    ES_U32 keylen;
    ES_U8 d[RSA2048_PRIKEY_LEN];
    ES_U8 mod[RSA2048_PRIKEY_LEN];
} rsa2048_privkey_t;

typedef enum
{
    OP_ENCRYPT = 0,
    OP_DECRYPT
}ES_CIPHER_CRYPTO_TYPE_E;

typedef enum esES_CIPHER_MEN_FLAG_E {
	USER_MALLOC = 0,
	HEAP_ALLOC,
}ES_CIPHER_MEN_FLAG_E;

static inline void print_data(ES_U8 *pData, ES_U32 size)
{
	ES_U32 i=0, j=0;
	for (i = 0; i < size/8; i++) {
		printf("[%03d]0x:%02x %02x %02x %02x %02x %02x %02x %02x\n", i*8,
			pData[i*8+0], pData[i*8+1], pData[i*8+2], pData[i*8+3],
			pData[i*8+4], pData[i*8+5], pData[i*8+6], pData[i*8+7]);
	}
	if (size%8)
		printf("[%03d]0x:", i*8);
	for (j = 0; j < size%8; j++)
		printf("%02x ", pData[i*8+j]);

	printf("\n");

}

static inline int alloc_buf_and_fill_data(void *src, int data_size, ES_U8 **ppVaddr, ES_U64 *pMemFd, ES_CIPHER_MEN_FLAG_E memFlag)
{
	int ret = 0;
	ES_U64 memFd;
	ES_U8 *pVaddr = NULL;

	if (memFlag == USER_MALLOC) {
		pVaddr = malloc(data_size);
		// pVaddr = g_buffer;
		if (pVaddr == NULL) {
			printf("%s, failed to alloc mem\n", __func__);
			return ret;
		}
		memFd = -1;
	}
	else {
		ret = ES_SYS_MemAlloc(&memFd, SYS_CACHE_MODE_CACHED, "encrypt", "mmz_nid_0_part_0", data_size);
		if (ES_SUCCESS != ret) {
			printf("%s, failed to alloc mem, err(%d)\n", __func__, ret);
			return ret;
		}

		pVaddr = ES_SYS_Mmap(memFd, data_size, SYS_CACHE_MODE_CACHED);
		if (NULL == pVaddr) {
			printf("%s, failed to mmap mem\n", __func__);
			ES_SYS_MemFree(memFd);
			return -1;
		}
	}

	if (NULL == src) {
		memset(pVaddr, 0, data_size);
	}
	else {
		memcpy(pVaddr, src, data_size);
	}

#ifndef ES_CIPHER_QEMU_DEBUG
	if (memFlag == HEAP_ALLOC) {
		ES_SYS_MemFlushCache (memFd);
	}
#endif
	*ppVaddr = pVaddr;
	*pMemFd = memFd;
	return 0;
}

static inline int free_data_buf(ES_U64 memFd, ES_U8 *pVaddr, int data_size)
{
	int ret = 0;

	if (memFd == -1) {
		free(pVaddr);
	}
	else {

		ret = ES_SYS_Munmap(pVaddr, data_size);
		if (ret != ES_SUCCESS) {
			printf("%s, failed to munmap:0x%p\n", __func__, pVaddr);
			return -1;
		}

		ret = ES_SYS_MemFree(memFd);
		if (ret != ES_SUCCESS) {
			printf("%s, failed to MemFree memFd:0x%llx\n", __func__, memFd);
			return -1;
		}

	}
	return 0;
}

static inline int file_size(char* filename)
{
	FILE *fp=fopen(filename,"r");
	if(!fp) return -1;
	fseek(fp,0L,SEEK_END);
	int size=ftell(fp);
	fclose(fp);

	return size;
}

static inline int file_read(FILE *file, unsigned char *data, uint64_t seek, int size)
{
	int read_size = 0;

	if ((file == NULL) || (data == NULL)) return -1;

	fseeko(file, seek, SEEK_SET);
	read_size = fread(data, sizeof(unsigned char), size, file);
	if (read_size < size)
	{
		if (!feof(file))
		{
			fprintf(stderr,"Err, read_size=0x%x, expected_size=0x%x, reach to the end of file!!!", read_size, size);
		}
		return -1;
	}

	printf("OK, read_size=0x%x, expected_size=0x%x\n", read_size, size);

	return 0;
}

static inline int read_file_to_buf(char *filename, void *buf)
{
	int size = 0;
	int ret = 0;
	FILE *fp = fopen(filename, "rb");

	printf("read_file_to_buf, filename:%s\n", filename);
	if(fp == NULL) {
		printf("Failed to open %s!!!\n", filename);
		return -1;
	}

	size = file_size(filename);//file_size2(filename);
	if (size < 0) {
		printf("failed to get file size!!!\n");
		fclose(fp);
		return -1;
	}

	printf("read_file_to_buf, expected size=0x%x\n", size);
	ret = file_read(fp, buf, 0, size);
	fclose(fp);

	return ret;
}
#endif
